@@ -130,8 +130,8 @@ setMethod("Solver.get_problem_data", "Solver", function(solver, objective, const
 
   conv_idx <- Solver._noncvx_id_to_idx(data[[DIMS]], sym_data@.var_offsets, sym_data@.var_sizes)
   data[[DIMS]] <- conv_idx$dims
-  data[[BOOL_IDX]] <- conv_idx$bool_idx
-  data[[INT_IDX]] <- conv_idx$int_idx
+  data[[BOOL_IDX]] <- unique(conv_idx$bool_idx)
+  data[[INT_IDX]] <- unique(conv_idx$int_idx)
   data
 })
 
@@ -404,9 +404,11 @@ setMethod("Solver.solve", "ECOS_BB", function(solver, objective, constraints, ca
   data <- Solver.get_problem_data(solver, objective, constraints, cached_data)
 
   # TODO: Naras please fix this by making ECOSolveR handle type conversion, e.g. logical -> integer
-  if(prod(dim(data[[G_KEY]])) == 0) data[[G_KEY]] <- NULL
-  if(prod(dim(data[[A_KEY]])) == 0) data[[A_KEY]] <- NULL
-  data[[DIMS]] <- lapply(data[[DIMS]], function(dim) { as.integer(dim) })
+  # TODO: Naras please fix this by making ECOSolveR handle type conversion, e.g. logical -> integer
+  ## Naras Answer: Fixed in ECOSolveR version >= 0.4
+  ## if(prod(dim(data[[G_KEY]])) == 0) data[[G_KEY]] <- NULL
+  ## if(prod(dim(data[[A_KEY]])) == 0) data[[A_KEY]] <- NULL
+  ## data[[DIMS]] <- lapply(data[[DIMS]], function(dim) { as.integer(dim) })
   storage.mode(data[[BOOL_IDX]]) <- "integer"
   storage.mode(data[[INT_IDX]]) <- "integer"
 
@@ -741,344 +743,4 @@ SCS.tri_to_full <- function(lower_tri, n) {
 #   new_results
 # })
 
-## #'
-## #' The MOSEK class.
-## #'
-## #' This class is an interface for the commercial MOSEK solver.
-## #'
-## #' @references E. Andersen and K. Andersen. "The MOSEK Interior Point Optimizer for Linear Programming: an Implementation of the Homogeneous Algorithm." \emph{High Performance Optimization}, vol. 33, pp. 197-232, 2000.
-## #' @seealso the \href{https://www.mosek.com/products/mosek/}{MOSEK Official Site}.
-## #' @name MOSEK-class
-## #' @aliases MOSEK
-## #' @rdname MOSEK-class
-## setClass("MOSEK", contains = "Solver")
 
-## #' @rdname MOSEK-class
-## #' @export
-## MOSEK <- function() {
-##   stop("Unimplemented")
-##   new("MOSEK")
-## }
-
-# #' @param object,solver A \linkS4class{MOSEK} object.
-# #' @describeIn MOSEK MOSEK can handle linear programs.
-# setMethod("lp_capable", "MOSEK", function(solver) { TRUE })
-#
-# #' @describeIn MOSEK MOSEK can handle second-order cone programs.
-# setMethod("socp_capable", "MOSEK", function(solver) { TRUE })
-#
-# #' @describeIn MOSEK MOSEK can handle semidefinite programs.
-# setMethod("sdp_capable", "MOSEK", function(solver) { TRUE })
-#
-# #' @describeIn MOSEK MOSEK cannot handle exponential cone programs.
-# setMethod("exp_capable", "MOSEK", function(solver) { FALSE })
-#
-# #' @describeIn MOSEK MOSEK cannot handle mixed-integer programs.
-# setMethod("mip_capable", "MOSEK", function(solver) { FALSE })
-#
-# #'
-# #' MOSEK Status Map
-# #'
-# #' Map of MOSEK status to CVXR status.
-# #'
-# #' @param solver A \linkS4class{MOSEK} object.
-# #' @param status An exit code returned by MOSEK. See the \href{http://docs.mosek.com/8.0/dotnetfusion/solution_status.html}{MOSEK documentation} for details.
-# #' @return A string indicating the status, either "optimal", "infeasible", "unbounded", "optimal_inaccurate", "infeasible_inaccurate", "unbounded_inaccurate", or "solver_error".
-# #' @docType methods
-# #' @rdname MOSEK-status_map
-# setMethod("status_map", "MOSEK", function(solver, status) {
-#   if(status == "OPTIMAL")
-#     OPTIMAL
-#   else if(status == "PRIMAL_INFEASIBLE_CER")
-#     INFEASIBLE
-#   else if(status == "DUAL_INFEASIBLE_CER")
-#     UNBOUNDED
-#   else if(status == "NEAR_OPTIMAL")
-#     OPTIMAL_INACCURATE
-#   else if(status == "NEAR_PRIMAL_INFEASIBLE_CER")
-#     INFEASIBLE_INACCURATE
-#   else if(status == "NEAR_DUAL_INFEASIBLE_CER")
-#     UNBOUNDED_INACCURATE
-#   else
-#     SOLVER_ERROR
-# })
-#
-# #' @describeIn MOSEK The name of the solver.
-# #' @export
-# setMethod("name", "MOSEK", function(object) { MOSEK_NAME })
-#
-# #' @describeIn MOSEK Imports the Rmosek library.
-# setMethod("import_solver", "MOSEK", function(solver) { requireNamespace("Rmosek") })
-#
-# setMethod("split_constr", "MOSEK", function(solver, constr_map) {
-#   list(eq_constr = constr_map[[EQ_MAP]], ineq_constr = constr_map[[LEQ_MAP]], nonlin_constr = list())
-# })
-#
-# #' @param objective A list representing the canonicalized objective.
-# #' @param constraints A list of canonicalized constraints.
-# #' @param cached_data A list mapping solver name to cached problem data.
-# #' @param warm_start A logical value indicating whether the previous solver result should be used to warm start.
-# #' @param verbose A logical value indicating whether to print solver output.
-# #' @param ... Additional arguments to the solver.
-# #' @describeIn MOSEK Call the solver on the canonicalized problem.
-# setMethod("Solver.solve", "MOSEK", function(solver, objective, constraints, cached_data, warm_start, verbose, ...) {
-#   data <- Solver.get_problem_data(solver, objective, constraints, cached_data)
-#
-#   A <- data[[A_KEY]]
-#   b <- data[[B_KEY]]
-#   G <- data[[G_KEY]]
-#   h <- data[[H_KEY]]
-#   c <- data[[C_KEY]]
-#   dims <- data[[DIMS]]
-#   problem <- list(sense = "minimize")
-#
-#   # Size of problem
-#   numvar <- length(c) + sum(dims[[SOC_DIM]])
-#   numcon <- length(b) + dims[[LEQ_DIM]] + sum(dims[[SOC_DIM]]) + sum(dims[[SDP_DIM]]^2)
-#
-#   # TODO: Fix crash on empty problem
-#
-#   # Objective
-#   problem$c <- c      # Objective coefficients
-#   problem$c0 <- 0     # Objective constant
-#   problem$bx <- rbind(blx = rep(-Inf, numvar), bux = rep(Inf, numvar))   # Lower and upper variable bounds
-#
-#   # SDP variables
-#   if(sum(dims[[SDP_DIM]]) > 0)
-#     problem$bardim <- dims[[SDP_DIM]]   # Semidefinite variable dimensions
-#
-#   # Linear equality and linear inequality constraints
-#   if(nrow(A) > 0 && nrow(G) > 0)
-#     constraints_matrix <- rbind(A, G)
-#   else if(nrow(A) > 0)
-#     constraints_matrix <- A
-#   else
-#     constraints_matrix <- G
-#   if(dims[[LEQ_DIM]] == length(h))
-#     problem$bc <- rbind(blc = c(b, rep(-Inf, dims[[LEQ_DIM]])), buc = c(b, h))
-#   else
-#     problem$bc <- rbind(blc = c(b, rep(-Inf, dims[[LEQ_DIM]]), h[(1 + dims[[LEQ_DIM]]):length(h)]), buc = c(b, h))
-#   # h_leq <- h[1:dims[[LEQ_DIM]]]
-#   # sdp_total_dims <- sum(dims[[SDP_DIM]]^2)
-#   # soc_sdp_dims <- sum(dims[[SOC_DIM]]) + sdp_total_dims
-#   # h_soc_sdp <- h[(1 + dims[[LEQ_DIM]]):(1 + dims[[LEQ_DIM]] + soc_sdp_dims)]
-#   # problem$bc <- rbind(blc = c(b, rep(-Inf, dims[[LEQ_DIM]]), h_soc_sdp), buc = c(b, h_leq, h_soc_sdp))
-#
-#   # Cone constraints
-#   num_cones <- length(dims[[SOC_DIM]])
-#   if(num_cones > 0) {
-#     cur_var_idx <- length(c)
-#     cur_con_idx <- length(b) + dims[[LEQ_DIM]]
-#     cones <- matrix(list(), nrow = 2, ncol = num_cones)
-#
-#     for(k in 1:num_cones) {
-#       size_cone <- dims[[SOC_DIM]][k]
-#
-#       # Add an identity for each cone
-#       id_mat <- sparseMatrix(i = cur_con_idx + 1:size_cone, j = cur_var_idx + 1:size_cone, x = 1)
-#       constraints_matrix <- rbind(constraints_matrix, id_mat)
-#
-#       # Add a cone constraint
-#       cones[,k] <- list("QUAD", seq(cur_var_idx + 1, cur_var_idx + size_cone))
-#       cur_var_idx <- cur_var_idx + size_cone
-#       cur_con_idx <- cur_con_idx + size_cone
-#     }
-#     rownames(cones) <- c("type", "sub")
-#     problem$cones <- cones
-#   }
-#   problem$A <- constraints_matrix
-#
-#   # SDP constraints
-#   num_sdp <- length(dims[[SDP_DIM]])
-#   if(num_sdp > 0) {
-#     for(k in 1:num_sdp) {
-#       size_matrix <- dims[[SDP_DIM]][k]
-#       for(i in 1:length(size_matrix)) {
-#         for(j in 1:length(size_matrix)) {
-#           if(i == j)
-#             coeff <- 1
-#           else
-#             coeff <- 0.5
-#           # TODO: Finish this
-#           cur_con_idx <- cur_con_idx + 1
-#         }
-#       }
-#     }
-#   }
-#
-#   results_dict <- Rmosek::mosek(problem, opts = list(getinfo = TRUE, soldetail = TRUE, verbose = verbose, ...))
-#   format_results(solver, results_dict, data, cached_data)
-# })
-#
-# #' @param results_dict A list containing the solver output.
-# #' @describeIn MOSEK Chooses between the basic and interior point solution.
-# setMethod("choose_solution", "MOSEK", function(solver, results_dict) {
-#   rank <- function(status) {
-#     # Rank solutions: optimal > near_optimal > anything else > None
-#     if(status == "OPTIMAL")
-#       return(3)
-#     else if(status == "NEAR_OPTIMAL")
-#       return(2)
-#     else if(!is.null(status))
-#       return(1)
-#     else
-#       return(0)
-#   }
-#
-#   # As long as interior solution is not worse, take it (for backward compatibility)
-#   solsta_bas <- results_dict$bas$solsta
-#   solsta_itr <- results_dict$itr$solsta
-#   if(rank(solsta_itr) >= rank(solsta_bas))
-#     list(solist = results_dict$sol$itr, solsta = solsta_itr)
-#   else
-#     list(solist = results_dict$sol$bas, solsta = solsta_bas)
-# })
-#
-# #' @param data A list containing information about the problem.
-# #' @describeIn MOSEK Convert raw solver output into standard list of results.
-# setMethod("format_results", "MOSEK", function(solver, results_dict, data, cached_data) {
-#   sol <- choose_solution(solver, results_dict)
-#
-#   new_results <- list()
-#   new_results[[STATUS]] <- status_map(sol$solsta)
-#   new_results[[SOLVE_TIME]] <- results_dict$dinfo$OPTIMIZER_TIME
-#   new_results[[SETUP_TIME]] <- results_dict$dinfo$PRESOLVE_TIME
-#   new_results[[NUM_ITERS]] <- results_dict$iinfo$INTPNT_ITER
-#
-#   if(new_results[[STATUS]] %in% SOLUTION_PRESENT) {
-#     # Get primal variable values
-#     new_results[[PRIMAL]] <- sol$solist$xx
-#
-#     # Get objective value
-#     new_results[[VALUE]] <- sol$solist$pobjval + data[[OFFSET]]
-#
-#     # TODO: Check if signs are inverted in MOSEK
-#     y <- sol$solist$slc - sol$solist$suc
-#     new_results[[EQ_DUAL]] <- y[1:length(data[[B_KEY]])]
-#     if(length(data[[B_KEY]]) < length(y))
-#       new_results[[INEQ_DUAL]] <- y[(length(data[[B_KEY]]) + 1):(length(data[[B_KEY]]) + data[[DIMS]][[LEQ_DIM]])]
-#   }
-#   new_results
-# })
-
-## #'
-## #' The GUROBI class.
-## #'
-## #' This class is an interface for the commercial GUROBI solver.
-## #'
-## #' @references \emph{Gurobi optimizer reference manual version 5.0,} Gurobi Optimization, Inc., Houston, Texas, July 2012.
-## #' @seealso the \href{http://www.gurobi.com/documentation/7.5/refman/r_api_overview.html}{GUROBI Official Site}.
-## #' @name GUROBI-class
-## #' @aliases GUROBI
-## #' @rdname GUROBI-class
-## setClass("GUROBI", contains = "Solver")
-
-## #' @rdname GUROBI-class
-## #' @export
-## GUROBI <- function() {
-##   stop("Unimplemented")
-##   new("GUROBI")
-## }
-
-# #' @param object,solver A \linkS4class{GUROBI} object.
-# #' @describeIn GUROBI GUROBI can handle linear programs.
-# setMethod("lp_capable", "GUROBI", function(solver) { TRUE })
-#
-# #' @describeIn GUROBI GUROBI can handle second-order cone programs.
-# setMethod("socp_capable", "GUROBI", function(solver) { TRUE })
-#
-# #' @describeIn GUROBI GUROBI cannot handle semidefinite programs.
-# setMethod("sdp_capable", "GUROBI", function(solver) { FALSE })
-#
-# #' @describeIn GUROBI GUROBI cannot handle exponential cone programs.
-# setMethod("exp_capable", "GUROBI", function(solver) { FALSE })
-#
-# #' @describeIn GUROBI GUROBI can handle mixed-integer programs.
-# setMethod("mip_capable", "GUROBI", function(solver) { TRUE })
-#
-# #'
-# #' GUROBI Status Map
-# #'
-# #' Map of GUROBI status to CVXR status.
-# #'
-# #' @param solver A \linkS4class{GUROBI} object.
-# #' @param status An exit code returned by GUROBI. See the \href{http://www.gurobi.com/documentation/7.5/refman/optimization_status_codes.html}{GUROBI documentation} for details.
-# #' @return A string indicating the status, either "optimal", "infeasible", "unbounded", "optimal_inaccurate", "infeasible_inaccurate", "unbounded_inaccurate", or "solver_error".
-# #' @docType methods
-# #' @rdname GUROBI-status_map
-# setMethod("status_map", "GUROBI", function(solver, status) {
-#   if(status == 2)
-#     OPTIMAL
-#   else if(status == 3)
-#     INFEASIBLE
-#   else if(status == 5)
-#     UNBOUNDED
-#   else if(status == 9)
-#     OPTIMAL_INACCURATE
-#   else if(status %in% c(4, 6, 7, 8, 10, 11, 12, 13))
-#     SOLVER_ERROR
-#   else
-#     stop("GUROBI status unrecognized: ", status)
-# })
-#
-# #' @describeIn GUROBI The name of the solver.
-# #' @export
-# setMethod("name", "GUROBI", function(object) { GUROBI_NAME })
-#
-# #' @describeIn GUROBI Imports the gurobi library.
-# setMethod("import_solver", "GUROBI", function(solver) { requireNamespace("gurobi") })
-#
-# setMethod("split_constr", "GUROBI", function(solver, constr_map) {
-#   list(eq_constr = c(constr_map[[EQ_MAP]], constr_map[[LEQ_MAP]]), ineq_constr = list(), nonlin_constr = list())
-# })
-#
-# #' @param results_dict A list containing the solver output.
-# #' @param data A list containing information about the problem.
-# #' @param cached_data A list mapping solver name to cached problem data.
-# #' @describeIn GUROBI Convert raw solver output into standard list of results.
-# setMethod("format_results", "GUROBI", function(solver, results_dict, data, cached_data) {
-#   dims <- data[[DIMS]]
-#   if(results_dict$status != SOLVER_ERROR) {
-#     solver_cache <- cached_data[[name(solver)]]
-#     solver_cache@prev_result <- list(vbasis = results_dict$vbasis, cbasis = results_dict$cbasis, c = data[[C_KEY]], A = data[[A_KEY]], b = data[[B_KEY]])
-#   }
-#
-#   new_results <- list()
-#   new_results[[STATUS]] <- results_dict$status
-#   new_results[[SOLVE_TIME]] <- results_dict$runtime
-#   if(new_results[[STATUS]] %in% SOLUTION_PRESENT) {
-#     primal_val <- results_dict$objval
-#     new_results[[VALUE]] <- primal_val + data[[OFFSET]]
-#     new_results[[PRIMAL]] <- results_dict$x
-#     if(!Solver.is_mip(data)) {
-#       duals <- results_dict$pi
-#       if(dims[[EQ_DIM]] > 0)
-#         new_results[[EQ_DUAL]] <- duals[1:dims[[EQ_DIM]]]
-#       if(length(duals) > dims[[EQ_DIM]])
-#         new_results[[INEQ_DUAL]] <- duals[(dims[[EQ_DIM]] + 1):length(duals)]
-#     }
-#   }
-# })
-
-####################
-#                  #
-# Solver utilities #
-#                  #
-####################
-# solver_intf <- list(ECOS(), ECOS_BB(), CVXOPT(), GLPK(), GLPK_MI(), CBC(), SCS(), GUROBI(), Elemental(), MOSEK(), LS())
-solver_intf <- list(ECOS(), ECOS_BB(), SCS())
-SOLVERS <- solver_intf
-names(SOLVERS) <- sapply(solver_intf, function(solver) { name(solver) })
-
-#'
-#' Installed Solvers
-#'
-#' @return The names of all the installed solvers.
-#' @docType methods
-#' @rdname installed_solvers
-#' @export
-installed_solvers <- function() {
-  installed <- sapply(SOLVERS, is_installed)
-  names(SOLVERS)[which(installed)]
-
-}
